/**
 *  \file IMP/container/PairsRestraint.h
 *  \brief Apply a PairScore to each Pair in a list.
 *
 *  WARNING This file was generated from PairsRestraint.h
 *  in /builddir/build/BUILD/imp-2.2.0/imp/tools/build/container_templates/container
 *  by tools/maintenance/setup_containers.py.
 *
 *  Copyright 2007-2014 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPCONTAINER_PAIRS_RESTRAINT_H
#define IMPCONTAINER_PAIRS_RESTRAINT_H

#include <IMP/container/container_config.h>
#include <IMP/kernel/internal/ContainerRestraint.h>
#include <IMP/kernel/PairContainer.h>
#include <IMP/kernel/PairScore.h>

IMPCONTAINER_BEGIN_NAMESPACE

//! Applies a PairScore to each Pair in a list.
/** This restraint stores the used particles in a kernel::ParticlePairsTemp.
    The container used can be set so that the list can be shared
    with other containers (or a nonbonded list can be used).

    Examples using various multiplicity containers:
    \include restrain_in_sphere.py
    \include nonbonded_interactions.py

    \see IMP::core::PairRestraint
 */
class PairsRestraint :
#if defined(SWIG) || defined(IMP_DOXYGEN)
    public kernel::Restraint
#else
    public IMP::kernel::internal::ContainerRestraint<kernel::PairScore,
                                                     kernel::PairContainer>
#endif
    {
  typedef IMP::kernel::internal::ContainerRestraint<
      kernel::PairScore, kernel::PairContainer> P;

 public:
  //! Create the restraint with a shared container
  /** \param[in] ss The function to apply to each particle.
      \param[in] pc The container containing the stored particles. This
      container is not copied.
      \param[in] name The object name
   */
  PairsRestraint(kernel::PairScore *ss,
                      kernel::PairContainerAdaptor pc,
                      std::string name = "PairsRestraint %1%")
      : P(ss, pc, name) {}

#if defined(IMP_DOXYGEN) || defined(SWIG)
  double unprotected_evaluate(IMP::kernel::DerivativeAccumulator *accum) const;
  IMP::kernel::ModelObjectsTemp do_get_inputs() const;
  IMP_OBJECT_METHODS(PairsRestraint)
#endif
};

IMP_OBJECTS(PairsRestraint, PairsRestraints);

IMPCONTAINER_END_NAMESPACE

#endif /* IMPCONTAINER_PAIRS_RESTRAINT_H */
