/**
 *  \file IMP/container/QuadsRestraint.h
 *  \brief Apply a QuadScore to each Quad in a list.
 *
 *  WARNING This file was generated from QuadsRestraint.h
 *  in /builddir/build/BUILD/imp-2.2.0/imp/tools/build/container_templates/container
 *  by tools/maintenance/setup_containers.py.
 *
 *  Copyright 2007-2014 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPCONTAINER_QUADS_RESTRAINT_H
#define IMPCONTAINER_QUADS_RESTRAINT_H

#include <IMP/container/container_config.h>
#include <IMP/kernel/internal/ContainerRestraint.h>
#include <IMP/kernel/QuadContainer.h>
#include <IMP/kernel/QuadScore.h>

IMPCONTAINER_BEGIN_NAMESPACE

//! Applies a QuadScore to each Quad in a list.
/** This restraint stores the used particles in a kernel::ParticleQuadsTemp.
    The container used can be set so that the list can be shared
    with other containers (or a nonbonded list can be used).

    Examples using various multiplicity containers:
    \include restrain_in_sphere.py
    \include nonbonded_interactions.py

    \see IMP::core::QuadRestraint
 */
class QuadsRestraint :
#if defined(SWIG) || defined(IMP_DOXYGEN)
    public kernel::Restraint
#else
    public IMP::kernel::internal::ContainerRestraint<kernel::QuadScore,
                                                     kernel::QuadContainer>
#endif
    {
  typedef IMP::kernel::internal::ContainerRestraint<
      kernel::QuadScore, kernel::QuadContainer> P;

 public:
  //! Create the restraint with a shared container
  /** \param[in] ss The function to apply to each particle.
      \param[in] pc The container containing the stored particles. This
      container is not copied.
      \param[in] name The object name
   */
  QuadsRestraint(kernel::QuadScore *ss,
                      kernel::QuadContainerAdaptor pc,
                      std::string name = "QuadsRestraint %1%")
      : P(ss, pc, name) {}

#if defined(IMP_DOXYGEN) || defined(SWIG)
  double unprotected_evaluate(IMP::kernel::DerivativeAccumulator *accum) const;
  IMP::kernel::ModelObjectsTemp do_get_inputs() const;
  IMP_OBJECT_METHODS(QuadsRestraint)
#endif
};

IMP_OBJECTS(QuadsRestraint, QuadsRestraints);

IMPCONTAINER_END_NAMESPACE

#endif /* IMPCONTAINER_QUADS_RESTRAINT_H */
