/**
 *  \file IMP/container/TripletsRestraint.h
 *  \brief Apply a TripletScore to each Triplet in a list.
 *
 *  WARNING This file was generated from TripletsRestraint.h
 *  in /builddir/build/BUILD/imp-2.2.0/imp/tools/build/container_templates/container
 *  by tools/maintenance/setup_containers.py.
 *
 *  Copyright 2007-2014 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPCONTAINER_TRIPLETS_RESTRAINT_H
#define IMPCONTAINER_TRIPLETS_RESTRAINT_H

#include <IMP/container/container_config.h>
#include <IMP/kernel/internal/ContainerRestraint.h>
#include <IMP/kernel/TripletContainer.h>
#include <IMP/kernel/TripletScore.h>

IMPCONTAINER_BEGIN_NAMESPACE

//! Applies a TripletScore to each Triplet in a list.
/** This restraint stores the used particles in a kernel::ParticleTripletsTemp.
    The container used can be set so that the list can be shared
    with other containers (or a nonbonded list can be used).

    Examples using various multiplicity containers:
    \include restrain_in_sphere.py
    \include nonbonded_interactions.py

    \see IMP::core::TripletRestraint
 */
class TripletsRestraint :
#if defined(SWIG) || defined(IMP_DOXYGEN)
    public kernel::Restraint
#else
    public IMP::kernel::internal::ContainerRestraint<kernel::TripletScore,
                                                     kernel::TripletContainer>
#endif
    {
  typedef IMP::kernel::internal::ContainerRestraint<
      kernel::TripletScore, kernel::TripletContainer> P;

 public:
  //! Create the restraint with a shared container
  /** \param[in] ss The function to apply to each particle.
      \param[in] pc The container containing the stored particles. This
      container is not copied.
      \param[in] name The object name
   */
  TripletsRestraint(kernel::TripletScore *ss,
                      kernel::TripletContainerAdaptor pc,
                      std::string name = "TripletsRestraint %1%")
      : P(ss, pc, name) {}

#if defined(IMP_DOXYGEN) || defined(SWIG)
  double unprotected_evaluate(IMP::kernel::DerivativeAccumulator *accum) const;
  IMP::kernel::ModelObjectsTemp do_get_inputs() const;
  IMP_OBJECT_METHODS(TripletsRestraint)
#endif
};

IMP_OBJECTS(TripletsRestraint, TripletsRestraints);

IMPCONTAINER_END_NAMESPACE

#endif /* IMPCONTAINER_TRIPLETS_RESTRAINT_H */
