/**
 *  \file IMP/core/PairConstraint.h
 *  \brief Use a PairModifier applied to a kernel::ParticlePairsTemp to
 *  maintain an invariant
 *
 *  This file is generated by a script (core/tools/make-container).
 *  Do not edit directly.
 *
 *  Copyright 2007-2014 IMP Inventors. All rights reserved.
 */

#ifndef IMPCORE_PAIR_CONSTRAINT_H
#define IMPCORE_PAIR_CONSTRAINT_H

#include <IMP/core/core_config.h>
#include <IMP/kernel/internal/TupleConstraint.h>
#include <IMP/PairModifier.h>
#include <IMP/PairDerivativeModifier.h>

IMPCORE_BEGIN_NAMESPACE
//! Apply a PairFunction to a Pair
/** The score state is passed up to two PairModifiers, one to
    apply before evaluation and the other after. The one after
    should take a DerivativeAccumulator as its last argument for
    PairModifier::apply() and will only be called if
    the score was computed with derivatives.

    \see container::PairsConstraint
 */
class PairConstraint :
#if defined(IMP_DOXYGEN) || defined(SWIG)
    public Constraint
#else
    public IMP::kernel::internal::TupleConstraint<PairModifier,
                                                  PairDerivativeModifier>
#endif
    {
 public:
  /** \deprecated_at{2.1} Use the model/index constructor.
   */
  IMPCORE_DEPRECATED_METHOD_DECL(2.1)
  PairConstraint(PairModifier *before,
                      PairDerivativeModifier *after, const kernel::ParticlePair& vt,
                      std::string name = "PairConstraint %1%")
      : IMP::kernel::internal::TupleConstraint<
            PairModifier, PairDerivativeModifier>(before, after, vt,
                                                            name) {
    IMPCORE_DEPRECATED_METHOD_DEF(2.1, "Use the model/index constructor.");
  }

  PairConstraint(PairModifier *before,
                      PairDerivativeModifier *after, kernel::Model *m,
                      const kernel::ParticleIndexPair& vt,
                      std::string name = "PairConstraint %1%")
      : IMP::kernel::internal::TupleConstraint<
            PairModifier, PairDerivativeModifier>(before, after, m,
                                                            vt, name) {}

#if defined(IMP_DOXYGEN) || defined(SWIG)
 protected:
  void do_update_attributes();
  void do_update_derivatives(DerivativeAccumulator *da);
  virtual kernel::ModelObjectsTemp do_get_inputs() const;
  virtual kernel::ModelObjectsTemp do_get_outputs() const;
  IMP_OBJECT_METHODS(PairConstraint);
#endif
};

IMPCORE_END_NAMESPACE

#endif /* IMPCORE_PAIR_CONSTRAINT_H */
