/**
 *  \file IMP/kernel/QuadPredicate.h
 *  \brief Define QuadPredicate.
 *
 *  WARNING This file was generated from QuadPredicate.h
 *  in /builddir/build/BUILD/imp-2.2.0/imp/tools/build/container_templates/kernel
 *  by tools/maintenance/setup_containers.py.
 *
 *  Copyright 2007-2014 IMP Inventors. All rights reserved.
 */

#ifndef IMPKERNEL_QUAD_PREDICATE_H
#define IMPKERNEL_QUAD_PREDICATE_H

#include <IMP/kernel/kernel_config.h>
#include "base_types.h"
#include "ParticleTuple.h"
#include "DerivativeAccumulator.h"
#include "internal/container_helpers.h"
#include "model_object_helpers.h"

IMPKERNEL_BEGIN_NAMESPACE

//! Abstract predicate function
/** A predicate is a function which returns one of a discrete set of
    values (eg -1, 0, 1 depending on whether a value is negative, zero
    or positive). QuadPredicates will evaluate the predicate for the passed
    particles.

    Implementers should check out IMP_QUAD_PREDICATE().
*/
class IMPKERNELEXPORT QuadPredicate : public ParticleInputs,
                                           public base::Object {
 public:
  typedef kernel::ParticleQuad Argument;
  typedef kernel::ParticleIndexQuad IndexArgument;
  QuadPredicate(std::string name = "QuadPredicate %1%");
  /** \deprecated_at{2.1} Use the index based version.*/
  IMPKERNEL_DEPRECATED_METHOD_DECL(2.1)
  virtual int get_value(const kernel::ParticleQuad& vt) const;

  /** \deprecated_at{2.1} Use the index based version.*/
  IMPKERNEL_DEPRECATED_METHOD_DECL(2.1)
  virtual Ints get_value(const kernel::ParticleQuadsTemp &o) const;

#if !defined(IMP_DOXYGEN) && !defined(SWIG)
  virtual void remove_if_equal(kernel::Model *m, kernel::ParticleIndexQuads &ps,
                               int v) const;
  virtual void remove_if_not_equal(kernel::Model *m, kernel::ParticleIndexQuads &ps,
                                   int v) const;
#endif

  //! Compute the predicate and the derivative if needed.
  virtual int get_value_index(kernel::Model *m, const kernel::ParticleIndexQuad& vt) const;

  //! Enable them to be use as functors
  /** But beware of slicing.
   */
  int operator()(kernel::Model *m, const kernel::ParticleIndexQuad& vt) const {
    return get_value_index(m, vt);
  }

  virtual Ints get_value_index(kernel::Model *m,
                               const kernel::ParticleIndexQuads &o) const {
    Ints ret(o.size());
    for (unsigned int i = 0; i < o.size(); ++i) {
      ret[i] += get_value_index(m, o[i]);
    }
    return ret;
  }

  IMP_REF_COUNTED_DESTRUCTOR(QuadPredicate);
};

IMPKERNEL_END_NAMESPACE

#endif /* IMPKERNEL_QUAD_PREDICATE_H */
