/**
 *  \file IMP/kernel/SingletonScore.h
 *  \brief Define SingletonScore.
 *
 *  WARNING This file was generated from SingletonScore.h
 *  in /builddir/build/BUILD/imp-2.2.0/imp/tools/build/container_templates/kernel
 *  by tools/maintenance/setup_containers.py.
 *
 *  Copyright 2007-2014 IMP Inventors. All rights reserved.
 */

#ifndef IMPKERNEL_SINGLETON_SCORE_H
#define IMPKERNEL_SINGLETON_SCORE_H

#include <IMP/kernel/kernel_config.h>
#include "base_types.h"
#include "ParticleTuple.h"
#include "DerivativeAccumulator.h"
#include "internal/container_helpers.h"
#include <IMP/base/utility_macros.h>
#include "model_object_helpers.h"

IMPKERNEL_BEGIN_NAMESPACE

//! Abstract score function
/** SingletonScores will evaluate the score and derivatives
    for the passed particles. Use in conjunction with various
    restraints such as IMP::core::SingletonsRestraint or
    IMP::core::SingletonRestraint.

    Implementers should check out IMP_SINGLETON_SCORE().
*/
class IMPKERNELEXPORT SingletonScore : public ParticleInputs,
                                       public base::Object {
 public:
  typedef kernel::Particle* Argument;
  typedef kernel::ParticleIndex IndexArgument;
  typedef kernel::Particle* PassArgument;
  typedef kernel::ParticleIndex PassIndexArgument;
  typedef SingletonModifier Modifier;
  SingletonScore(std::string name = "SingletonScore %1%");
  //! Compute the score and the derivative if needed.
  /** \deprecated_at{2.1} Use the index-based evaluate instead. */
  IMPKERNEL_DEPRECATED_METHOD_DECL(2.1)
  virtual double evaluate(kernel::Particle* vt, DerivativeAccumulator *da) const;

  //! Compute the score and the derivative if needed.
  virtual double evaluate_index(kernel::Model *m, kernel::ParticleIndex vt,
                                DerivativeAccumulator *da) const;

  /** Implementations
      for these are provided by the IMP_SINGLETON_SCORE()
      macro.
  */
  virtual double evaluate_indexes(kernel::Model *m, const kernel::ParticleIndexes &o,
                                  DerivativeAccumulator *da,
                                  unsigned int lower_bound,
                                  unsigned int upper_bound) const;

  //! Compute the score and the derivative if needed.
  virtual double evaluate_if_good_index(kernel::Model *m, kernel::ParticleIndex vt,
                                        DerivativeAccumulator *da,
                                        double max) const;

  /** Implementations
      for these are provided by the IMP_SINGLETON_SCORE()
      macro.
  */
  virtual double evaluate_if_good_indexes(kernel::Model *m,
                                          const kernel::ParticleIndexes &o,
                                          DerivativeAccumulator *da, double max,
                                          unsigned int lower_bound,
                                          unsigned int upper_bound) const;
  /** Decompose this pair score acting on the pair into a set of
      restraints. The scoring function and derivatives should
      be equal to the current score. The defualt implementation
      just returns this object bound to the pair.*/
  Restraints create_current_decomposition(kernel::Model *m,
                                          kernel::ParticleIndex vt) const;

 protected:
  /** Overide this to return your own decomposition.*/
  virtual Restraints do_create_current_decomposition(kernel::Model *m,
                                                     kernel::ParticleIndex vt) const;

  IMP_REF_COUNTED_DESTRUCTOR(SingletonScore);
};

IMPKERNEL_END_NAMESPACE

#endif /* IMPKERNEL_SINGLETON_SCORE_H */
